/*  DMX Eingenbau Moving Head V1.0
*
* Title			: DMX512 Empfang
* Last updated	: 21.04.07
* Target		: MH-1 [AtMega32]
* Clock			: 8MHz, 16MHz

**************************************************************************/
 
//Prprozessor-Anweisungen
#include "dmx512.h"

//Locale Definitionen 
enum {IDLE, BREAK, STARTB, STARTADR};			//DMX states

		 uint8_t 	 gDmxState;
		 uint8_t 	*gDmxPnt;
		 uint16_t	 DmxCount;


// *************** DMX Empfang Initialisation ****************
void init_DMX(void)
{
	/* #ifdef USE_DIP
	DDRC= 0;										//set up DIPs
	PORTC= 0xFF;
	DDRE &= ~((1<<2)|(1<<1));
	PORTE |= (1<<2)|(1<<1);
	#endif */

	DDRD |= (1<<2);
	PORTD &= ~(1<<2);								//enable reception
	UBRRH  = 0;
	UBRRL  = ((F_OSC/4000)-1);						//250kBaud
	UCSRB |= (1<<RXEN)|(1<<RXCIE);					//RX Enable; RX Complete Interrupt Enable
	UCSRC |= (1<<URSEL)		//UCSRC writing enable
		  | (1<<UCSZ0)		//8-bit
		  | (1<<UCSZ1)
		  | (1<<USBS);		//2 Stop-Bits

	gDmxState= IDLE;

	uint8_t i;
	for (i=0; i<DMX_CHANNELS; i++)
		DmxField[i]= 0;
		

}


// ************* get DMX start address **************
/*void get_dips(void)
{
 #ifdef USE_DIP
uint16_t Temp= ~PINC & 0xFF; //invert DIP state
if (!(PINE &(1<<2)))
	{
	Temp= Temp +256;							//9th bit
	}
if (Temp != 0)
	{
	DmxAddress= Temp;
	if (!(UCSRB &(1<<RXCIE)))					//if receiver was disabled -> enable and wait for break
		{
		gDmxState= IDLE;
		UCSRB |= (1<<RXCIE);
		}
	}
else
	{
	UCSRB &= ~(1<<RXCIE);						//disable Receiver if start address = 0
	for (Temp=0; Temp<DMX_CHANNELS; Temp++)
		{
		DmxField[Temp]= 0;
		}
	}
#endif 
} */


// *************** DMX Reception ISR ****************
ISR (USART_RXC_vect)
{
	uint8_t USARTstate= UCSRA;						//get state
	uint8_t DmxByte= UDR;							//get data
	uint8_t DmxState= gDmxState;					//just get once from SRAM!!!
	DmxAddress = 1;
 
	if (USARTstate &(1<<FE))						//check for break
	{
		UCSRA &= ~(1<<FE);							//reset flag
		DmxCount= DmxAddress;						//reset frame counter
		gDmxState= BREAK;
	}

	else if (DmxState == BREAK)
	{
		if (DmxByte == 0) 
			{
				gDmxState= STARTB;						//normal start code detected
				gDmxPnt= ((uint8_t*)DmxField +1);
			}
		else gDmxState= IDLE;
	}
	
	else if (DmxState == STARTB)
	{
		if (--DmxCount == 0)						//start address reached?
			{
				gDmxState= STARTADR;
				DmxField[0]= DmxByte;
			}
	}

	else if (DmxState == STARTADR)
	{
		uint8_t *DmxPnt;
		DmxPnt= gDmxPnt;
		*DmxPnt= DmxByte;
		if (++DmxPnt >= (DmxField +DMX_CHANNELS)) 	//all ch received?
		{
			gDmxState= IDLE;
		}
		else gDmxPnt= DmxPnt;
	}							
}


